# 导入SQLite驱动:
import sqlite3
# 连接到SQLite数据库
# 数据库文件是hedron.db
# 如果文件不存在，会自动在当前目录创建:
from hedronagent.util.logger import logger
# @Singleton
class ConnectDatabase():
    def __init__(self):
        self.conn = sqlite3.connect('/usr/hedron/hedronagent/database/hedron.db')
        # 创建一个Cursor:
        cursor = self.conn.cursor()
        cursor.execute('''create table if not exists messages 
            (uuid TEXT primary key, 
            time TEXT,
            urgency INTEGER, 
            sender TEXT, 
            title TEXT, 
            content TEXT, 
            status INTEGER)''')
        self.conn.commit()
        cursor.close()
        # self.cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
        # tables = [v[0] for v in self.cursor.fetchall() if v[0] != "sqlite_sequence"]
        # print(tables)
        # # 执行一条SQL语句，创建user表:
        # if "messages" not in tables:
        #     self.cursor.execute('''create if not exists table messages
        #         (uuid TEXT primary key,
        #         urgency INTEGER,
        #         sender TEXT,
        #         title TEXT,
        #         content TEXT,
        #         status INTEGER)''')
        #     logger.info("成功建立messages表")
        # else:
        #     logger.info("messages表已建立，不需要重建！")
        logger.info("Message database is ready")

    def insert_messages_table(self, message):
        temp = (message['uuid'], message['time'], message['urgency'], message['sender'], message['title'], message['content'], message['status'])
        try:
            # # 插入数据
            # for t in [('1', 'alex', 'man', '189'), ('2', 'tom', 'man', '139')]:
            #     conn.execute('INSERT INTO customer VALUES(?,?,?,?)', t)
            # 继续执行一条SQL语句，插入一条记录:
            cursor = self.conn.cursor()
            cursor.execute('insert into messages values (?, ?, ?, ?, ?, ?, ?)', temp)
            # self.cursor.execute('insert into user (telephone, name, department, unit, address, fixedphone) values (' + telephone + name + department + unit + address + fixedphone+')')
            # 提交事务:
            self.conn.commit()
            logger.info("Successfully insert {} row(s)".format(cursor.rowcount))
            cursor.close()
            return 0
        except Exception as e:
            # 出错，回滚
            self.conn.rollback()
            logger.exception(e)
            return -1
        # 通过rowcount获得插入的行数:

    def query_message_number(self):
        cursor = self.conn.cursor()
        cursor.execute("SELECT count(*) FROM messages;")
        number = cursor.fetchone()[0]
        cursor.close()
        logger.info("The total number of messages is {}".format(number))
        return number

    def query_page_messages(self, pageSize, page):
        cursor = self.conn.cursor()
        cursor.execute("SELECT * FROM messages ORDER BY time DESC limit {} offset {}".format(pageSize, pageSize * (page - 1)))
        # cursor.execute(f"SELECT * FROM messages ORDER BY ROWID DESC limit {pageSize} offset {pageSize}*{page - 1}")
        results = cursor.fetchall()
        cursor.close()
        messages = []
        for result in results:
            messages.append({'uuid': result[0],
                        'time': result[1],
                        'urgency': result[2],
                        'sender': result[3],
                        'title': result[4],
                        'content': result[5],
                        'status': result[6]})
        return messages

    def query_the_message(self, pageSize, page, index):
        cursor = self.conn.cursor()
        cursor.execute("SELECT * FROM messages ORDER BY time DESC limit 1 offset {0};".format(pageSize * (page - 1) + index) )
        # cursor.execute(f"SELECT * FROM messages ORDER BY ROWID DESC limit 1 offset {pageSize * (page - 1) + index};", )
        result = cursor.fetchone()
        cursor.close()
        message = {'uuid': result[0],
                        'time': result[1],
                        'urgency': result[2],
                        'sender': result[3],
                        'title': result[4],
                        'content': result[5],
                        'status': result[6]}
        return message

    def query_message_status_by_uuid(self, uuid):
        cursor = self.conn.cursor()
        cursor.execute("SELECT * FROM messages where uuid='{0}';".format(uuid))
        # cursor.execute(f"SELECT * FROM messages where uuid='{uuid}';")
        result = cursor.fetchone()
        cursor.close()
        return result[6]

    def query_unreceived_messages(self):
        cursor = self.conn.cursor()
        cursor.execute("SELECT count(*) FROM messages;")
        messages = []
        for i in range(cursor.fetchone()[0]):
            cursor.execute("SELECT * FROM messages where status=0 ORDER BY ROWID DESC limit 10 offset 10*{0}".format(i))
            # cursor.execute(f"SELECT * FROM messages where status=0 ORDER BY ROWID DESC limit 10 offset 10*{i}")
            results = cursor.fetchall()
            if results == []:
                break
            for result in results:
                messages.append({'uuid': result[0],
                            'time': result[1],
                            'urgency': result[2],
                            'sender': result[3],
                            'title': result[4],
                            'content': result[5],
                            'status': result[6]})
        cursor.close()
        if messages == []:
            logger.info("No message to be received")
        else:
            logger.info("Get message(s) to be received: {}！".format(messages))
        return messages

    def update_message_status(self, uuid, status):
        cursor = self.conn.cursor()
        cursor.execute("update messages set status=? where uuid=?;",(status, uuid))
        self.conn.commit()
        if cursor.rowcount == 1:
            logger.info("Update the message status of {} to {}".format(uuid, status))
            return 0
        else:
            logger.error("Fail to update the message status of {}.".format(uuid))
        cursor.close()
        return -1
